////////////////////////////////////////////////////////////////////////////////
// checkstyle: Checks Java source code for adherence to a set of rules.
// Copyright (C) 2001-2012  Oliver Burn
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
package com.puppycrawl.tools.checkstyle.checks.indentation;

import com.puppycrawl.tools.checkstyle.BaseCheckTestSupport;
import com.puppycrawl.tools.checkstyle.Checker;
import com.puppycrawl.tools.checkstyle.DefaultConfiguration;
import org.junit.Test;

/**
 *
 * @author  jrichard
 */
public class IndentationCheckTest extends BaseCheckTestSupport
{
    @Test
    public void testInvalidLabel() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final String[] expected = {
            "20:17: label child at indentation level 10 not at correct indentation, 8",
            "24:17: label child at indentation level 16 not at correct indentation, 12",
            "29:17: label child at indentation level 2 not at correct indentation, 4",
            "32:17: label child at indentation level 18 not at correct indentation, 8",
            "33:17: ctor def child at indentation level 18 not at correct indentation, 8",
            "35:17: label child at indentation level 6 not at correct indentation, 8",
            "35:17: method call child at indentation level 6 not at correct indentation, 8",
            "37:17: label child at indentation level 6 not at correct indentation, 8",
            "37:17: method call child at indentation level 6 not at correct indentation, 8",
        };
        verify(checkConfig, getPath("indentation/InputInvalidLabelIndent.java"), expected);
    }

    @Test
    public void testValidLabel() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final String[] expected = {
        };
        verify(checkConfig, getPath("indentation/InputValidLabelIndent.java"), expected);
    }

    @Test
    public void testValidIfWithChecker() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidIfIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidDotWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidDotIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidMethodWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidMethodIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidMethodWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidMethodIndent.java");
        final String[] expected = {
            "17:17: ctor def rcurly at indentation level 6 not at correct indentation, 4",
            "20:17: ctor def modifier at indentation level 6 not at correct indentation, 4",
            "21:17: ctor def lcurly at indentation level 2 not at correct indentation, 4",
            "22:17: ctor def rcurly at indentation level 6 not at correct indentation, 4",
            "25:17: method def modifier at indentation level 2 not at correct indentation, 4",
            "26:17: method def rcurly at indentation level 6 not at correct indentation, 4",
            "63:17: method def modifier at indentation level 5 not at correct indentation, 4",
            "64:17: method def modifier at indentation level 5 not at correct indentation, 4",
            "65:17: method def return type at indentation level 5 not at correct indentation, 4",
            "74:17: method def modifier at indentation level 3 not at correct indentation, 4",
            "75:17: method def modifier at indentation level 3 not at correct indentation, 4",
            "76:17: method def return type at indentation level 3 not at correct indentation, 4",
            "77:17: method def at indentation level 5 not at correct indentation, 4",
            "87:17: ctor def child at indentation level 4 not at correct indentation, 8",
            "87:17: method call child at indentation level 4 not at correct indentation, 8",
            "92:17: method def child at indentation level 6 not at correct indentation, 8",
            "93:17: if at indentation level 6 not at correct indentation, 8",
            "94:17: if child at indentation level 10 not at correct indentation, 12",
            "94:17: method call child at indentation level 10 not at correct indentation, 12",
            "95:17: if rcurly at indentation level 6 not at correct indentation, 8",
            "98:17: method call child at indentation level 10 not at correct indentation, 12",
            "104:17: method call child at indentation level 14 not at correct indentation, 16",
            "107:17: method call child at indentation level 10 not at correct indentation, 12",
            "112:17: method call child at indentation level 14 not at correct indentation, 16",
            "116:17: method call child at indentation level 10 not at correct indentation, 12",
            "120:17: method call child at indentation level 10 not at correct indentation, 12",
            "121:17: method call rparen at indentation level 6 not at correct indentation, 8",
            "125:17: method call rparen at indentation level 6 not at correct indentation, 8",
            "139:17: method call child at indentation level 10 not at correct indentation, 12",
            "142:17: method call child at indentation level 10 not at correct indentation, 12",
            "152:17: method call child at indentation level 6 not at correct indentation, 12",
            "158:17: method def throws at indentation level 6 not at correct indentation, 8",
            "164:17: method def child at indentation level 4 not at correct indentation, 8",
            "169:17: method def child at indentation level 4 not at correct indentation, 8",
            "173:17: method def return type at indentation level 0 not at correct indentation, 4",
        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidSwitchWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidSwitchIndent.java");
        final String[] expected = {
            "26:17: switch at indentation level 6 not at correct indentation, 8",
            "28:17: case child at indentation level 10 not at correct indentation, 12",
            "29:17: block child at indentation level 14 not at correct indentation, 16",
            "29:17: method call child at indentation level 14 not at correct indentation, 16",
            "33:17: block child at indentation level 14 not at correct indentation, 16",
            "35:17: case child at indentation level 14 not at correct indentation, 12",
            "36:17: case child at indentation level 10 not at correct indentation, 12",
            "39:17: case child at indentation level 10 not at correct indentation, 12",
            "40:17: block child at indentation level 14 not at correct indentation, 16",
            "40:17: method call child at indentation level 14 not at correct indentation, 16",
            "41:17: block child at indentation level 14 not at correct indentation, 16",
            "49:17: block child at indentation level 14 not at correct indentation, 16",
            "49:17: method call child at indentation level 14 not at correct indentation, 16",
            "50:17: block child at indentation level 18 not at correct indentation, 16",
            "51:17: block rcurly at indentation level 10 not at correct indentation, 12",
            "55:17: block lcurly at indentation level 10 not at correct indentation, 12",
            "58:17: block rcurly at indentation level 14 not at correct indentation, 12",
            "62:17: block lcurly at indentation level 14 not at correct indentation, 12",
            "65:17: block rcurly at indentation level 10 not at correct indentation, 12",
            "72:17: case child at indentation level 14 not at correct indentation, 16",
            "77:17: case child at indentation level 14 not at correct indentation, 16",
            "85:17: switch rcurly at indentation level 6 not at correct indentation, 8",
            "88:17: switch lcurly at indentation level 6 not at correct indentation, 8",
            "89:17: switch rcurly at indentation level 10 not at correct indentation, 8",
            "91:17: switch lcurly at indentation level 10 not at correct indentation, 8",
            "92:17: switch rcurly at indentation level 6 not at correct indentation, 8",
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidSwitchWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidSwitchIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidArrayInitWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidArrayInitIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidArrayInitWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidArrayInitIndent.java");
        final String[] expected = {
            "17:17: member def type at indentation level 2 not at correct indentation, 4",
            "18:17: member def type at indentation level 6 not at correct indentation, 4",
            "20:17: member def type at indentation level 2 not at correct indentation, 4",
            "22:17: array initialization rcurly at indentation level 6 not at correct indentation, 2",
            "24:17: member def type at indentation level 6 not at correct indentation, 4",
            "25:17: array initialization child at indentation level 8 not at correct indentation, 10",
            "26:17: array initialization rcurly at indentation level 4 not at correct indentation, 6",
            "29:17: array initialization child at indentation level 9 not at correct indentation, 8",
            "30:17: array initialization child at indentation level 7 not at correct indentation, 8",
            "31:17: array initialization child at indentation level 9 not at correct indentation, 8",
            "36:17: array initialization lcurly at indentation level 2 not at correct indentation, 4",
            "40:17: array initialization rcurly at indentation level 6 not at correct indentation, 4",
            "44:17: array initialization lcurly at indentation level 2 not at correct indentation, 4",
            "48:17: array initialization child at indentation level 20 not at correct indentation, [8, 31]",
            "49:17: array initialization child at indentation level 4 not at correct indentation, [8, 31]",
            "54:17: array initialization child at indentation level 6 not at correct indentation, 8",
            "59:17: member def type at indentation level 2 not at correct indentation, 4",
            "61:17: member def type at indentation level 6 not at correct indentation, 4",
            "62:17: array initialization rcurly at indentation level 2 not at correct indentation, 6",
            "65:17: array initialization child at indentation level 6 not at correct indentation, 8",
            "72:17: array initialization child at indentation level 10 not at correct indentation, 12",
            "85:17: array initialization child at indentation level 8 not at correct indentation, 12",
            "96:17: array initialization child at indentation level 10 not at correct indentation, 12",
            "97:17: array initialization child at indentation level 14 not at correct indentation, 12",
            "100:17: array initialization child at indentation level 10 not at correct indentation, 12",
            "101:17: array initialization child at indentation level 14 not at correct indentation, 12",
            "102:17: array initialization rcurly at indentation level 6 not at correct indentation, 8",
            "105:17: array initialization lcurly at indentation level 6 not at correct indentation, 8",
            "106:17: array initialization child at indentation level 14 not at correct indentation, 12",
            "107:17: array initialization child at indentation level 10 not at correct indentation, 12",
            "108:17: array initialization rcurly at indentation level 6 not at correct indentation, 8",
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidTryWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidTryIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidTryWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidTryIndent.java");
        final String[] expected = {
            "21:17: try at indentation level 9 not at correct indentation, 8",
            "22:17: try rcurly at indentation level 7 not at correct indentation, 8",
            "24:17: catch rcurly at indentation level 7 not at correct indentation, 8",
            "26:17: try at indentation level 4 not at correct indentation, 8",
            "27:17: method call child at indentation level 8 not at correct indentation, 12",
            "27:17: try child at indentation level 8 not at correct indentation, 12",
            "28:17: try rcurly at indentation level 4 not at correct indentation, 8",
            "29:17: finally child at indentation level 8 not at correct indentation, 12",
            "29:17: method call child at indentation level 8 not at correct indentation, 12",
            "34:17: catch child at indentation level 8 not at correct indentation, 12",
            "34:17: method call child at indentation level 8 not at correct indentation, 12",
            "39:17: try rcurly at indentation level 10 not at correct indentation, 8",
            "41:17: catch rcurly at indentation level 6 not at correct indentation, 8",
            "48:17: catch rcurly at indentation level 5 not at correct indentation, 8",
            "55:17: catch child at indentation level 10 not at correct indentation, 12",
            "55:17: method call child at indentation level 10 not at correct indentation, 12",
            "56:17: catch child at indentation level 14 not at correct indentation, 12",
            "57:17: catch child at indentation level 10 not at correct indentation, 12",
            "57:17: method call child at indentation level 10 not at correct indentation, 12",
            "59:17: catch at indentation level 6 not at correct indentation, 8",
            "66:17: try lcurly at indentation level 10 not at correct indentation, 8",
            "68:17: try rcurly at indentation level 10 not at correct indentation, 8",
            "70:17: catch lcurly at indentation level 6 not at correct indentation, 8",
            "73:17: catch rcurly at indentation level 10 not at correct indentation, 8",
            "76:17: catch child at indentation level 10 not at correct indentation, 12",
            "76:17: method call child at indentation level 10 not at correct indentation, 12",
        };
        verify(c, fname, expected);
    }

    // TODO: needs to be finished
    @Test
    public void testInvalidClassDefWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidClassDefIndent.java");
        final String[] expected = {
            "19:17: class def ident at indentation level 2 not at correct indentation, 0",
            "19:17: class def modifier at indentation level 2 not at correct indentation, 0",
            "25:17: class def lcurly at indentation level 2 not at correct indentation, 0",
            "28:17: class def rcurly at indentation level 2 not at correct indentation, 0",
            "31:17: class def ident at indentation level 2 not at correct indentation, 0",
            "35:17: class def rcurly at indentation level 2 not at correct indentation, 0",
            "40:17: class def child at indentation level 2 not at correct indentation, 4",
            "41:17: class def child at indentation level 2 not at correct indentation, 4",
            "47:17: class def child at indentation level 2 not at correct indentation, 4",
            "55:17: class def child at indentation level 2 not at correct indentation, 4",
            "56:17: class def child at indentation level 2 not at correct indentation, 4",
            "61:17: class def ident at indentation level 2 not at correct indentation, 0",
            "61:17: class def modifier at indentation level 2 not at correct indentation, 0",
            "62:17: class def lcurly at indentation level 2 not at correct indentation, 0",
            "70:17: class def rcurly at indentation level 2 not at correct indentation, 0",
            "74:17: class def child at indentation level 2 not at correct indentation, 4",
            "83:17: class def child at indentation level 2 not at correct indentation, 4",
            "83:17: class def ident at indentation level 2 not at correct indentation, 4",
            "85:17: class def ident at indentation level 6 not at correct indentation, 4",
            "88:17: class def child at indentation level 2 not at correct indentation, 4",
            "88:17: class def ident at indentation level 2 not at correct indentation, 4",
            "92:17: member def modifier at indentation level 6 not at correct indentation, 8",
            "93:17: member def type at indentation level 12 not at correct indentation, 8",
            "98:17: member def type at indentation level 10 not at correct indentation, 8",
            "103:17: member def modifier at indentation level 6 not at correct indentation, 8",
            "108:17: class def rcurly at indentation level 6 not at correct indentation, 4",
            "110:17: class def ident at indentation level 6 not at correct indentation, 4",
            "116:17: class def child at indentation level 6 not at correct indentation, 8",
            "116:17: class def ident at indentation level 6 not at correct indentation, 8",
            "119:17: class def ident at indentation level 10 not at correct indentation, 8",
            "121:17: class def rcurly at indentation level 10 not at correct indentation, 8",
            "124:17: member def type at indentation level 10 not at correct indentation, 12",
            "129:17: method def child at indentation level 10 not at correct indentation, 8",
            "130:17: object def lcurly at indentation level 8 not at correct indentation, [10, 14]",
            "134:17: object def rcurly at indentation level 8 not at correct indentation, [10, 14]",
            "138:17: object def lcurly at indentation level 6 not at correct indentation, [8, 12]",

            "139:17: method def modifier at indentation level 12 not at correct indentation, 10",
            "141:17: method def rcurly at indentation level 12 not at correct indentation, 10",

            "142:17: object def rcurly at indentation level 6 not at correct indentation, [8, 12]",
            "147:17: method def modifier at indentation level 10 not at correct indentation, 12",
            "149:17: method def rcurly at indentation level 10 not at correct indentation, 12",
            // TODO: extends and implements need to be added
        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidBlockWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidBlockIndent.java");
        final String[] expected = {
            "22:17: block lcurly at indentation level 7 not at correct indentation, 8",
            "23:17: block lcurly at indentation level 9 not at correct indentation, 8",
            "25:17: block lcurly at indentation level 9 not at correct indentation, 8",
            "26:17: block rcurly at indentation level 7 not at correct indentation, 8",
            "28:17: block lcurly at indentation level 6 not at correct indentation, 8",
            "30:17: block rcurly at indentation level 6 not at correct indentation, 8",
            "31:17: block lcurly at indentation level 6 not at correct indentation, 8",
            "34:17: block lcurly at indentation level 9 not at correct indentation, 8",
            "35:17: block child at indentation level 13 not at correct indentation, 12",
            "37:17: block child at indentation level 13 not at correct indentation, 12",
            "38:17: block rcurly at indentation level 9 not at correct indentation, 8",
            "41:17: block lcurly at indentation level 6 not at correct indentation, 8",
            "42:17: block child at indentation level 10 not at correct indentation, 12",
            "44:17: block child at indentation level 10 not at correct indentation, 12",
            "45:17: block rcurly at indentation level 6 not at correct indentation, 8",
            "48:17: block lcurly at indentation level 6 not at correct indentation, 8",
            "51:17: block child at indentation level 10 not at correct indentation, 12",
            "55:17: block lcurly at indentation level 10 not at correct indentation, 12",
            "59:17: block rcurly at indentation level 10 not at correct indentation, 12",
            "64:17: block child at indentation level 10 not at correct indentation, 12",
            "66:17: block lcurly at indentation level 10 not at correct indentation, 12",
            "67:17: block child at indentation level 14 not at correct indentation, 16",
            "82:17: block rcurly at indentation level 10 not at correct indentation, 12",
            "91:17: static initialization at indentation level 2 not at correct indentation, 4",
            "92:17: static initialization at indentation level 6 not at correct indentation, 4",
            "96:17: static initialization child at indentation level 7 not at correct indentation, 8",
            "99:17: static initialization at indentation level 6 not at correct indentation, 4",
            "101:17: static initialization rcurly at indentation level 2 not at correct indentation, 4",
            "103:17: static initialization at indentation level 2 not at correct indentation, 4",
            "105:17: static initialization rcurly at indentation level 6 not at correct indentation, 4",
            "107:17: static initialization at indentation level 2 not at correct indentation, 4",
            "109:17: static initialization child at indentation level 6 not at correct indentation, 8",
            "112:17: static initialization lcurly at indentation level 2 not at correct indentation, 4",
            "113:17: static initialization child at indentation level 6 not at correct indentation, 8",
            "114:17: static initialization rcurly at indentation level 6 not at correct indentation, 4",
            "119:17: static initialization child at indentation level 6 not at correct indentation, 8",
            "124:17: static initialization child at indentation level 4 not at correct indentation, 8",
            "125:17: static initialization rcurly at indentation level 2 not at correct indentation, 4",
            "130:17: static initialization rcurly at indentation level 6 not at correct indentation, 4",
            "133:17: block lcurly at indentation level 2 not at correct indentation, 4",
            "134:17: block lcurly at indentation level 6 not at correct indentation, 4",
            "137:17: block lcurly at indentation level 2 not at correct indentation, 4",
            "139:17: block rcurly at indentation level 6 not at correct indentation, 4",
            "141:17: block lcurly at indentation level 6 not at correct indentation, 4",
            "143:17: block rcurly at indentation level 2 not at correct indentation, 4",
            "146:17: block child at indentation level 6 not at correct indentation, 8",

        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidIfWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidIfIndent.java");
        final String[] expected = {
            "50:17: if at indentation level 1 not at correct indentation, 8",
            "55:17: if at indentation level 9 not at correct indentation, 8",
            "56:17: if lcurly at indentation level 9 not at correct indentation, 8",
            "57:17: if rcurly at indentation level 7 not at correct indentation, 8",
            "59:17: if at indentation level 6 not at correct indentation, 8",
            "60:17: if lcurly at indentation level 5 not at correct indentation, 8",
            "61:17: if rcurly at indentation level 5 not at correct indentation, 8",
            "65:17: if rcurly at indentation level 10 not at correct indentation, 8",
            "66:17: else rcurly at indentation level 7 not at correct indentation, 8",
            "69:17: if at indentation level 9 not at correct indentation, 8",
            "70:17: if lcurly at indentation level 7 not at correct indentation, 8",
            "72:17: else at indentation level 9 not at correct indentation, 8",
            "74:17: else rcurly at indentation level 9 not at correct indentation, 8",
            "77:17: if at indentation level 10 not at correct indentation, 8",
            "78:17: if rcurly at indentation level 7 not at correct indentation, 8",
            "79:17: else at indentation level 9 not at correct indentation, 8",
            "80:17: else lcurly at indentation level 7 not at correct indentation, 8",
            "81:17: else rcurly at indentation level 9 not at correct indentation, 8",
            "85:17: if at indentation level 9 not at correct indentation, 8",
            "86:17: if lcurly at indentation level 9 not at correct indentation, 8",
            "87:17: if rcurly at indentation level 9 not at correct indentation, 8",
            "88:17: else lcurly at indentation level 7 not at correct indentation, 8",
            "89:17: else rcurly at indentation level 10 not at correct indentation, 8",
            "92:17: if at indentation level 6 not at correct indentation, 8",
            "93:17: if lcurly at indentation level 10 not at correct indentation, 8",
            "94:17: if rcurly at indentation level 10 not at correct indentation, 8",
            "95:17: else rcurly at indentation level 7 not at correct indentation, 8",
            "98:17: if at indentation level 5 not at correct indentation, 8",
            "99:17: if rcurly at indentation level 11 not at correct indentation, 8",
            "100:17: else at indentation level 5 not at correct indentation, 8",
            "101:17: else rcurly at indentation level 11 not at correct indentation, 8",
//            "110: dot left side \"System\" at indentation level 14 not at correct indentation, 12",
            "121:17: if child at indentation level 14 not at correct indentation, 12",
            "126:17: if lcurly at indentation level 10 not at correct indentation, 8",
            "127:17: if child at indentation level 10 not at correct indentation, 12",
            "127:17: method call child at indentation level 10 not at correct indentation, 12",
            "132:17: if child at indentation level 14 not at correct indentation, 12",
            // TODO: should be 16, not 12
            "133:17: method call child at indentation level 10 not at correct indentation, 12",
            "135:17: else child at indentation level 10 not at correct indentation, 12",
            // TODO: why we get this message (it's duplicate of previous).
            "135:17: method call child at indentation level 10 not at correct indentation, 12",
            "136:17: method call child at indentation level 8 not at correct indentation, 12",
            "143:17: if child at indentation level 16 not at correct indentation, 12",
            "144:17: if rcurly at indentation level 9 not at correct indentation, 8",
            "147:17: else child at indentation level 16 not at correct indentation, 12",
            "153:17: if child at indentation level 0 not at correct indentation, 12",
            "153:17: method call child at indentation level 0 not at correct indentation, 12",
            "157:17: else child at indentation level 40 not at correct indentation, 12",
            "164:17: if child at indentation level 14 not at correct indentation, 12",
            "167:17: else child at indentation level 14 not at correct indentation, 12",
            "173:17: if child at indentation level 10 not at correct indentation, 12",
            "173:17: method call child at indentation level 10 not at correct indentation, 12",
            "175:17: else child at indentation level 10 not at correct indentation, 12",
            "175:17: method call child at indentation level 10 not at correct indentation, 12",
            "179:17: if at indentation level 10 not at correct indentation, 8",
            "180:17: if child at indentation level 14 not at correct indentation, 12",
            "181:17: if rcurly at indentation level 10 not at correct indentation, 8",
            "182:17: else at indentation level 10 not at correct indentation, 8",
            "183:17: else child at indentation level 14 not at correct indentation, 12",
            "184:17: else rcurly at indentation level 10 not at correct indentation, 8",
            "187:17: if child at indentation level 9 not at correct indentation, 12",
            "188:17: if child at indentation level 11 not at correct indentation, 12",
            "192:17: if child at indentation level 10 not at correct indentation, 12",
            "195:17: if rcurly at indentation level 7 not at correct indentation, 8",
            "202:17: if child at indentation level 10 not at correct indentation, 12",
            "202:17: method call child at indentation level 10 not at correct indentation, 12",
            "204:17: if child at indentation level 10 not at correct indentation, 12",
            "204:17: method call child at indentation level 10 not at correct indentation, 12",
            "211:17: method call child at indentation level 10 not at correct indentation, 12",
            "220:17: if at indentation level 10 not at correct indentation, 12",
            "224:17: if child at indentation level 18 not at correct indentation, 20",
            "224:17: method call child at indentation level 18 not at correct indentation, 20",
            "228:17: if rcurly at indentation level 40 not at correct indentation, 8",
            "235:17: if rparen at indentation level 10 not at correct indentation, 8",
            "240:17: if rparen at indentation level 6 not at correct indentation, 8",
            "246:17: if lparen at indentation level 6 not at correct indentation, 8",
            "248:17: if rparen at indentation level 6 not at correct indentation, 8",
        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidWhileWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidWhileIndent.java");
        final String[] expected = {
            "21:17: while at indentation level 9 not at correct indentation, 8",
            "22:17: while rcurly at indentation level 7 not at correct indentation, 8",
            "24:17: while at indentation level 7 not at correct indentation, 8",
            "25:17: while lcurly at indentation level 9 not at correct indentation, 8",
            "26:17: while rcurly at indentation level 9 not at correct indentation, 8",
            "28:17: while at indentation level 9 not at correct indentation, 8",
            "29:17: while lcurly at indentation level 6 not at correct indentation, 8",
            "30:17: while child at indentation level 14 not at correct indentation, 12",
            "31:17: while rcurly at indentation level 6 not at correct indentation, 8",
            "33:17: while at indentation level 10 not at correct indentation, 8",
            "35:17: while rcurly at indentation level 10 not at correct indentation, 8",
            "37:17: while at indentation level 10 not at correct indentation, 8",
            "40:17: while rcurly at indentation level 10 not at correct indentation, 8",
            "42:17: while at indentation level 6 not at correct indentation, 8",
            "43:17: while lcurly at indentation level 10 not at correct indentation, 8",
            "46:17: while rcurly at indentation level 6 not at correct indentation, 8",
            "49:17: if at indentation level 14 not at correct indentation, 12",
            "50:17: if child at indentation level 18 not at correct indentation, 16",
            "51:17: if rcurly at indentation level 14 not at correct indentation, 12",
            "52:17: while child at indentation level 14 not at correct indentation, 12",
            "53:17: while rcurly at indentation level 10 not at correct indentation, 8",
            "56:17: while child at indentation level 10 not at correct indentation, 12",
            "62:17: while child at indentation level 10 not at correct indentation, 12",
            "67:17: while child at indentation level 10 not at correct indentation, 12",
            "74:17: while rparen at indentation level 5 not at correct indentation, 8",
            "81:17: while rparen at indentation level 10 not at correct indentation, 8",
            "88:17: while rparen at indentation level 10 not at correct indentation, 8",
            "95:17: while child at indentation level 8 not at correct indentation, 12",
        };
        verify(c, fname, expected);
    }

    @Test
    public void testInvalidForWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputInvalidForIndent.java");
        final String[] expected = {
            "22:17: for at indentation level 6 not at correct indentation, 8",
            "23:17: for rcurly at indentation level 10 not at correct indentation, 8",
            "25:17: for at indentation level 9 not at correct indentation, 8",
            "26:17: for lcurly at indentation level 6 not at correct indentation, 8",
            "27:17: for rcurly at indentation level 6 not at correct indentation, 8",
            "31:17: for child at indentation level 10 not at correct indentation, 12",
            "31:17: method call child at indentation level 10 not at correct indentation, 12",
            "32:17: for rcurly at indentation level 10 not at correct indentation, 8",
            "35:17: for lcurly at indentation level 10 not at correct indentation, 8",
            "36:17: for child at indentation level 10 not at correct indentation, 12",
            "44:17: for child at indentation level 10 not at correct indentation, 12",
            "50:17: for at indentation level 7 not at correct indentation, 8",
            "51:17: for child at indentation level 10 not at correct indentation, 12",
            "60:17: for child at indentation level 7 not at correct indentation, 12",
            "65:17: for at indentation level 6 not at correct indentation, 8",
            "66:17: for child at indentation level 10 not at correct indentation, 12",
            "67:17: for child at indentation level 14 not at correct indentation, 16",
            "68:17: for child at indentation level 10 not at correct indentation, 12",
            "73:17: for rcurly at indentation level 39 not at correct indentation, 8",
            "77:17: for rparen at indentation level 12 not at correct indentation, 8",
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidForWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidForIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidDoWhileWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidDoWhileIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidBlockWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidBlockIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }


    @Test
    public void testValidWhileWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidWhileIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidClassDefWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidClassDefIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidInterfaceDefWithChecker()
        throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidInterfaceDefIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testValidCommaWithChecker()
        throws Exception
    {

        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final Checker c = createChecker(checkConfig);
        final String fname = getPath("indentation/InputValidCommaIndent.java");
        final String[] expected = {
        };
        verify(c, fname, expected);
    }

    @Test
    public void testTabs() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        checkConfig.addAttribute("basicOffset", Integer.valueOf(4).toString());
        checkConfig.addAttribute("tabWidth", Integer.valueOf(4).toString());
        final String[] expected = {
            "25:17: ctor def child at indentation level 9 not at correct indentation, 8",
        };
        verify(checkConfig, getPath("indentation/InputUseTabs.java"), expected);
    }

    @Test
    public void testIndentationLevel() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        checkConfig.addAttribute("basicOffset", Integer.valueOf(2).toString());
        final String[] expected = {
            "25:17: ctor def child at indentation level 5 not at correct indentation, 4",
        };
        verify(checkConfig, getPath("indentation/InputUseTwoSpaces.java"), expected);
    }

    @Test
    public void testCaseLevel() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        checkConfig.addAttribute("caseIndent", Integer.valueOf(0).toString());
        final String[] expected = {
            "23:17: case child at indentation level 10 not at correct indentation, 8",
        };
        verify(checkConfig, getPath("indentation/InputCaseLevel.java"), expected);
    }

    @Test
    public void testBraceAdjustment() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        checkConfig.addAttribute("braceAdjustment", Integer.valueOf(2).toString());
        final String[] expected = {
            "24:17: if rcurly at indentation level 8 not at correct indentation, 10",
        };
        verify(checkConfig, getPath("indentation/InputBraceAdjustment.java"), expected);
    }

    @Test
    public void testInvalidAssignWithChecker() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final String[] expected = {
            "6:17: method call child at indentation level 10 not at correct indentation, 12",
            "8:17: method call child at indentation level 10 not at correct indentation, 12",
            "10:17: method call child at indentation level 10 not at correct indentation, 12",
            "12:17: assign at indentation level 9 not at correct indentation, 12",
            "13:17: assign child at indentation level 10 not at correct indentation, 12",
        };
        verify(checkConfig, getPath("indentation/InputInvalidAssignIndent.java"), expected);
    }

    @Test
    public void testValidAssignWithChecker() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final String[] expected = {};
        verify(checkConfig, getPath("indentation/InputValidAssignIndent.java"), expected);
    }

    @Test
    public void test15Extensions() throws Exception
    {
        final DefaultConfiguration checkConfig = createCheckConfig(IndentationCheck.class);
        final String[] expected = {};
        verify(checkConfig, getPath("Input15Extensions.java"), expected);
    }

    @Test
    public void testTryResources() throws Exception
    {
        final DefaultConfiguration checkConfig =
            createCheckConfig(IndentationCheck.class);
        final String[] expected = {
            //"something is expected, but there is nothing",
        };
        verify(checkConfig, getPath("indentation/InputValidTryResourcesIndent.java"),
               expected);
    }
}
